unit Alakzatok;

interface

uses
  Graphics, SysUtils;

{Az alakzatok osztlyai}
type TPont=class(TObject)      {ha nem runk st, akkor a TObject az s}
     private
       {A pont koordinti}
       Fx,Fy:integer;
       {Szne}
       FSzin:TColor;
     public
       {Sajt Create a paramterek tvtelhez}
       constructor Create(Ax,Ay:integer; ASzin:TColor);
       {A Hova objektumra az x s y koordintknak megfelelen}
       procedure Kirak(AktSzin:TColor; Hova:TCanvas); virtual;
       {A Honnan objektumrl az x s y koordintknak megfelelen}
       procedure Levesz(Honnan: TCanvas);
       {A Hol objektumon mozgatunk dx,dy relatv koordintkkal}
       procedure Mozgat(dx,dy:integer; Hol:TCanvas);
       {Sajt Destroy a levtelhez}
       destructor Destroy(Hol:TCanvas); virtual;
       {A privt adatmezk elrse: tulajdonsgokkal}
       property x:integer read Fx write Fx;
       property y:integer read Fy write Fy;
       property Szin:TColor read FSzin write FSzin;
     end;

type TKor=class(TPont)
     private
       {A kr sugara (az rklt x,y a kr kzppontja)}
       FSugar:word;
       {A sugrt bellt metdus}
       procedure SetSugar(ASugar:word);
     public
       {Sajt Create}
       constructor Create(Ax,Ay:integer; ASugar:word; ASzin:TColor);
       {Az rklt (TPont.Kirak) virtulis metdust felldefiniljuk}
       procedure Kirak(AktSzin:TColor; Hova:TCanvas); override;
       {A privt adatmezk elrse: tulajdonsgokkal}
       property Sugar:word read FSugar write SetSugar;
     end;

type TTeliKor=class(TKor)
     public
       {Az rklt (TKor.Kirak) virtulis metdust felldefiniljuk}
       procedure Kirak(AktSzin:TColor; Hova:TCanvas); override;
     end;

type TTeglalap=class(TPont)
     private
       {Szlessg, magassg (az rklt x,y a bal fels sarok)}
       FSzel,FMag:word;
       {A szlessget, magassgot bellt metdusok}
       procedure SetSzel(ASzel:word);
       procedure SetMag(AMag:word);
     public
       {Sajt Create}
       constructor Create(Ax,Ay:integer; ASzel,AMag:word; ASzin:TColor);
       {Az rklt (TPont.Kirak) virtulis metdust felldefiniljuk}
       procedure Kirak(AktSzin:TColor; Hova:TCanvas); override;
       {A privt adatmezk elrse: tulajdonsgokkal}
       property Szel:word read FSzel write SetSzel;
       property Mag:word read FMag write SetMag;
     end;

type TTeliTeglalap=class(TTeglalap)
     public
       {Az rklt (TTeglalap.Kirak) virtulis metdust felldefiniljuk}
       procedure Kirak(AktSzin:TColor; Hova:TCanvas); override;
     end;

implementation

{A TPont osztly metdusai}
constructor TPont.Create(Ax,Ay:integer; ASzin:TColor);
begin
 {Az rklt Create (TObject.Create) meghvsa}
 inherited Create;
 {A koordintk, szn belltsa}
 x:=Ax; y:=Ay; szin:=ASzin;
end;

procedure TPont.Kirak(AktSzin:TColor; Hova:TCanvas);
begin
 Hova.Pixels[x,y]:=AktSzin;
end;

procedure TPont.Levesz(Honnan:TCanvas);
begin
 {A Honnan httrsznvel rajzoljuk ki}
 Kirak(Honnan.Brush.Color,Honnan);
end;

{A krt relatv koordintkkal arrbb rakjuk}
{A lemozgst a hv ellenrzi}
procedure TPont.Mozgat(dx,dy:integer; Hol:TCanvas);
begin
 Levesz(Hol);
 x:=x+dx; y:=y+dy;
 Kirak(Szin,Hol);
end;

destructor Tpont.Destroy(Hol:TCanvas);
begin
 Levesz(Hol);
 {Az rklt Destroy meghvsa}
 inherited Destroy;
end;

{A TKor osztly metdusai}
constructor TKor.Create(Ax,Ay:integer; ASugar:word; ASzin:TColor);
begin
 {A rklt Create (TPont.Create) meghvsa}
 inherited Create(Ax,Ay,Aszin);
 {A sugr belltsa}
 SetSugar(ASugar);
end;

procedure TKor.Kirak(AktSzin:TColor; Hova:TCanvas);
var sz:TColor;
begin
 {Az eredeti toll szn mentse}
 sz:=Hova.Pen.Color;
 {Az alakzatunk szne AktSzin legyen}
 Hova.Pen.Color:=AktSzin;
 {A befoglal tglalap kt tellenes sarkt kell megadni}
 Hova.Ellipse(x-Sugar,y-Sugar,x+Sugar,y+Sugar);
 {Az eredeti toll szn visszalltsa}
 Hova.Pen.Color:=sz;
end;

{A sugrt bellt metdus}
procedure TKor.SetSugar(ASugar:word);
begin
 if ASugar=0 then
  {A hibaobjektumot a rendszer megsznteti}
  raise ERangeError.Create('A sugr nem lehet 0!')
 else FSugar:=ASugar;
end;

{A TTeliKor osztly metdusai}
procedure TTeliKor.Kirak(AktSzin:TColor; Hova:TCanvas);
var psz,bsz:TColor;
begin
 {Az eredeti toll, ecset szn mentse}
 psz:=Hova.Pen.Color; bsz:=Hova.Brush.Color;
 {Az alakzatunk szne AktSzin legyen}
 Hova.Pen.Color:=AktSzin; Hova.Brush.Color:=AktSzin;
 {A befoglal tglalap kt tellenes sarkt kell megadni}
 Hova.Ellipse(x-Sugar,y-Sugar,x+Sugar,y+Sugar);
 {Az eredeti toll, ecset szn visszalltsa}
 Hova.Pen.Color:=psz; Hova.Brush.Color:=bsz;
end;

{A TTeglalap osztly metdusai}
constructor TTeglalap.Create(Ax,Ay:integer; ASzel,AMag:word; ASzin:TColor);
begin
 {A rklt Create (TPont.Create) meghvsa}
 inherited Create(Ax,Ay,Aszin);
 {A szlessg, magassg belltsa}
 SetSzel(ASzel); SetMag(AMag);
end;

{Az x,y pontot gy tekintjk, mint a tglalap bal fels sarkt}
procedure TTeglalap.Kirak(AktSzin:TColor; Hova:TCanvas);
var sz:TColor;
begin
 {Az eredeti toll szn mentse}
 sz:=Hova.Pen.Color;
 {Az alakzatunk szne AktSzin legyen}
 Hova.Pen.Color:=AktSzin;
 {A tglalap kt tellenes sarkt kell megadni}
 Hova.Rectangle(x,y,x+Szel,y+Mag);
 {Az eredeti toll szn visszalltsa}
 Hova.Pen.Color:=sz;
end;

{A szlessget bellt metdus}
procedure TTeglalap.SetSzel(ASzel:word);
begin
 if ASzel=0 then
  raise ERangeError.Create('A szlessg nem lehet 0!')
 else FSzel:=ASzel;
end;

{A magassgot bellt metdus}
procedure TTeglalap.SetMag(AMag:word);
begin
 if AMag=0 then
  raise ERangeError.Create('A magassg nem lehet 0!')
 else FMag:=AMag;
end;

{Az x,y pontot gy tekintjk, mint a tglalap bal fels sarkt}
procedure TTeliTeglalap.Kirak(AktSzin:TColor; Hova:TCanvas);
var psz,bsz:TColor;
begin
 {Az eredeti toll, ecset szn mentse}
 psz:=Hova.Pen.Color; bsz:=Hova.Brush.Color;
 {Az alakzatunk szne AktSzin legyen}
 Hova.Pen.Color:=AktSzin; Hova.Brush.Color:=AktSzin;
 {A tglalap kt tellenes sarkt kell megadni}
 Hova.Rectangle(x,y,x+Szel,y+Mag);
 {Az eredeti toll, ecset szn visszalltsa}
 Hova.Pen.Color:=psz; Hova.Brush.Color:=bsz;
end;

end.
